/*********************************************
 * OPL 12.6.2.0 Model
 * Author: aguinet
 * Creation Date: 13 oct. 2016 at 12:03:35
 *********************************************/
 /* Paramtres */
int Mzon = ...;
int Tper = ...;
int Ntre = ...;
int Nres =...;
int Couhosp =...;
int Couevac =...;
int Nbed =...;
float Tgv = 999999;

range I = 1..Ntre;
range P = 1..Tper;
range H = 1..Nres;
range J = 1..Mzon;

int Cap[H][I] = ...;
int Wpat[J] =...;
int Cou[H][I] =...;
int Nivm =...;
float Pert[J][P] =...;
int Freq[I] =...;

/* Variables de dcision */
dvar float+ Access [I][J][P];
dvar int+ X[H][I][J][P];
dvar boolean Y[J][P];
dvar float+ Z[J][P];
dvar float+ W[J][P]; 
dvar float+ V[J][P];
dvar float+ Cevac;
dvar float+ Cinf;
dvar float+ Cmed;

/* Objectif : minimiser les cots variables de maintient  domicile des patients 
et les cots fixes d'hospitaliastion */
minimize sum(h in H, i in I, j in J, p in P) (X[h][i][j][p] * Cou[h][i]) 
+ sum(j in J, p in P) (Y[j][p] * Wpat[j] * (Couhosp * (Tper-p+1) + Couevac));

/* Avec les contraintes */
subject to
{
  /* Etat initial avant la crise */
  forall(i in I, j in J)  Access [i][j][1] == 1;
  
  /*Evolution de la crise */
  forall(i in I, j in J, p in 1..Tper-1)  
  Access [i][j][p+1] == Access [i][j][p] + Pert [j][p]; 
   
  /* On vacue une zone au plus une seule fois */
  forall(j in J) sum(p in P) Y[j][p] <= 1;
  
  /* On soigne les patients  domicile ou on les hospitalise aprs vacuation */
  forall(i in I, j in J, p in P) sum (h in H) X[h][i][j][p] >= 
  (Wpat[j] * Access [i][j][p] * (Freq[i]/7) ) - (sum (q in 1..p) (Y[j][q] * Tgv)); 
  
  /* Lorsque la crue depasse le seuil maximum admissible Nivm, on vacue */
  forall(i in I, j in J, p in P) sum (q in 1..p) Y[j][q] * Tgv >= Access [i][j][p] - 1 - Nivm;
  
  /* On respecte les capacits journalires des ressources pour chaque type et pour chaque priode */
  forall(h in H, i in I, p in P) sum(j in J) X[h][i][j][p] <= Cap[h][i];
  
  /* On dispose d'un nombre de lits limit en structures d'accueil */
  sum (j in J, p in P) (Y [j][p] * Wpat[j]) <= Nbed;
  
  /* Indicateurs de performance : Nombre d'vacuations et nombre de visites infirmiers et mdecins */
  forall (j in J, p in P) Z [j][p] == Y [j][p] * Wpat[j]; 
  forall (j in J, p in P) W [j][p] == sum(h in H) X[h][1][j][p]; 
  forall (j in J, p in P) V [j][p] == sum(h in H) X[h][2][j][p]; 
  
  /* Cots vacuation, infirmiers et mdecins */
  Cevac == sum(j in J, p in P) (Y[j][p] * Wpat[j] * (Couhosp * (Tper-p+1) + Couevac));
  Cinf == sum(h in H, j in J, p in P) (X[h][1][j][p] * Cou[h][1]);
  Cmed == sum(h in H, j in J, p in P) (X[h][2][j][p] * Cou[h][2]);   
}
